@@ -9,10 +9,12 @@ Project Page: https://github.com/jasonrbriggs/stomp.py
 
 """
 
+import stomp.adapter as adapter
 import stomp.connect as connect
 import stomp.listener as listener
+import stomp.logging as logging
 
-__version__ = (4, 1, 19)
+__version__ = (6, 1, 0)
 
 ##
 # Alias for STOMP 1.0 connections.
