@@ -1,18 +1,16 @@
 """Various listeners for using with stomp.py connections.
 """
 
-import logging
+import os
 import sys
 import threading
 import time
+from time import monotonic
 
-from stomp.backward import monotonic
-from stomp.constants import *
 import stomp.exception as exception
 import stomp.utils as utils
-
-
-log = logging.getLogger('stomp.py')
+from stomp.constants import *
+from stomp import logging
 
 
 class Publisher(object):
@@ -66,7 +64,7 @@ class ConnectionListener(object):
         """
         pass
 
-    def on_connected(self, headers, body):
+    def on_connected(self, frame):
         """
         Called by the STOMP connection when a CONNECTED frame is
         received (after a connection has been established or
@@ -75,7 +73,6 @@ class ConnectionListener(object):
         :param dict headers: a dictionary containing all headers sent by the server as key/value pairs.
         :param body: the frame's payload. This is usually empty for CONNECTED frames.
         """
-        pass
 
     def on_disconnected(self):
         """
@@ -92,7 +89,7 @@ class ConnectionListener(object):
         """
         pass
 
-    def on_before_message(self, headers, body):
+    def on_before_message(self, frame):
         """
         Called by the STOMP connection before a message is returned to the client app. Returns a tuple
         containing the headers and body (so that implementing listeners can pre-process the content).
@@ -100,9 +97,9 @@ class ConnectionListener(object):
         :param dict headers: the message headers
         :param body: the message body
         """
-        return headers, body
+        pass
 
-    def on_message(self, headers, body):
+    def on_message(self, frame):
         """
         Called by the STOMP connection when a MESSAGE frame is received.
 
@@ -111,7 +108,7 @@ class ConnectionListener(object):
         """
         pass
 
-    def on_receipt(self, headers, body):
+    def on_receipt(self, frame):
         """
         Called by the STOMP connection when a RECEIPT frame is
         received, sent by the server if requested by the client using
@@ -122,7 +119,7 @@ class ConnectionListener(object):
         """
         pass
 
-    def on_error(self, headers, body):
+    def on_error(self, frame):
         """
         Called by the STOMP connection when an ERROR frame is received.
 
@@ -145,19 +142,28 @@ class ConnectionListener(object):
         """
         pass
 
+    def on_receiver_loop_completed(self, frame):
+        """
+        Called when the connection receiver_loop has finished.
+        """
+        pass
+
 
 class HeartbeatListener(ConnectionListener):
     """
     Listener used to handle STOMP heartbeating.
     """
-    def __init__(self, heartbeats):
+    def __init__(self, transport, heartbeats, heart_beat_receive_scale=1.5):
         self.running = False
+        self.transport = transport
         self.heartbeats = heartbeats
         self.received_heartbeat = None
         self.heartbeat_thread = None
         self.next_outbound_heartbeat = None
+        self.heart_beat_receive_scale = heart_beat_receive_scale
+        self.heartbeat_terminate_event = threading.Event()
 
-    def on_connected(self, headers, body):
+    def on_connected(self, frame):
         """
         Once the connection is established, and 'heart-beat' is found in the headers, we calculate the real
         heartbeat numbers (based on what the server sent and what was specified by the client) - if the heartbeats
@@ -166,14 +172,18 @@ class HeartbeatListener(ConnectionListener):
         :param dict headers: headers in the connection message
         :param body: the message body
         """
-        if 'heart-beat' in headers:
+        if "heart-beat" in frame.headers:
             self.heartbeats = utils.calculate_heartbeats(
-                headers['heart-beat'].replace(' ', '').split(','), self.heartbeats)
+                frame.headers["heart-beat"].replace(' ', '').split(','), self.heartbeats)
+            logging.debug("Heartbeats calculated %s", str(self.heartbeats))
             if self.heartbeats != (0, 0):
                 self.send_sleep = self.heartbeats[0] / 1000
 
-                # receive gets an additional grace of 50%
-                self.receive_sleep = (self.heartbeats[1] / 1000) * 1.5
+                # by default, receive gets an additional grace of 50%
+                # set a different heart-beat-receive-scale when creating the connection to override that
+                self.receive_sleep = (self.heartbeats[1] / 1000) * self.heart_beat_receive_scale
+
+                logging.debug("Set receive_sleep to %s, send_sleep to %s", self.receive_sleep, self.send_sleep)
 
                 # Give grace of receiving the first heartbeat
                 self.received_heartbeat = monotonic() + self.receive_sleep
@@ -187,8 +197,9 @@ class HeartbeatListener(ConnectionListener):
 
     def on_disconnected(self):
         self.running = False
+        self.heartbeat_terminate_event.set()
 
-    def on_message(self, headers, body):
+    def on_message(self, frame):
         """
         Reset the last received time whenever a message is received.
 
@@ -225,7 +236,7 @@ class HeartbeatListener(ConnectionListener):
         """
         if frame.cmd == CMD_CONNECT or frame.cmd == CMD_STOMP:
             if self.heartbeats != (0, 0):
-                frame.headers[HDR_HEARTBEAT] = '%s,%s' % self.heartbeats
+                frame.headers[HDR_HEARTBEAT] = "%s,%s" % self.heartbeats
         if self.next_outbound_heartbeat is not None:
             self.next_outbound_heartbeat = monotonic() + self.send_sleep
 
@@ -241,11 +252,13 @@ class HeartbeatListener(ConnectionListener):
         """
         Main loop for sending (and monitoring received) heartbeats.
         """
+        logging.info("Starting heartbeat loop")
         now = monotonic()
 
         # Setup the initial due time for the outbound heartbeat
         if self.send_sleep != 0:
             self.next_outbound_heartbeat = now + self.send_sleep
+            logging.debug("Calculated next outbound heartbeat as %s", self.next_outbound_heartbeat)
 
         while self.running:
             now = monotonic()
@@ -259,7 +272,9 @@ class HeartbeatListener(ConnectionListener):
                     next_events.append(t)
             sleep_time = min(next_events)
             if sleep_time > 0:
-                time.sleep(sleep_time)
+                terminate = self.heartbeat_terminate_event.wait(sleep_time)
+                if terminate:
+                    break
 
             now = monotonic()
 
@@ -268,21 +283,21 @@ class HeartbeatListener(ConnectionListener):
                 continue
 
             if self.send_sleep != 0 and now > self.next_outbound_heartbeat:
-                log.debug("Sending a heartbeat message at %s", now)
+                logging.debug("Sending a heartbeat message at %s", now)
                 try:
                     self.transport.transmit(utils.Frame(None, {}, None))
                 except exception.NotConnectedException:
-                    log.debug("Lost connection, unable to send heartbeat")
+                    logging.debug("Lost connection, unable to send heartbeat")
                 except Exception:
                     _, e, _ = sys.exc_info()
-                    log.debug("Unable to send heartbeat, due to: %s", e)
+                    logging.debug("Unable to send heartbeat, due to: %s", e)
 
             if self.receive_sleep != 0:
                 diff_receive = now - self.received_heartbeat
 
                 if diff_receive > self.receive_sleep:
                     # heartbeat timeout
-                    log.warning("Heartbeat timeout: diff_receive=%s, time=%s, lastrec=%s",
+                    logging.warning("Heartbeat timeout: diff_receive=%s, time=%s, lastrec=%s",
                                 diff_receive, now, self.received_heartbeat)
                     self.transport.set_connected(False)
                     self.transport.disconnect_socket()
@@ -290,6 +305,10 @@ class HeartbeatListener(ConnectionListener):
                     for listener in self.transport.listeners.values():
                         listener.on_heartbeat_timeout()
         self.heartbeat_thread = None
+        self.heartbeat_terminate_event.clear()
+        if self.heartbeats != (0, 0):
+            # don't bother logging this if heartbeats weren't setup to start with
+            logging.info("Heartbeat loop ended")
 
 
 class WaitingListener(ConnectionListener):
@@ -300,31 +319,45 @@ class WaitingListener(ConnectionListener):
         """
         :param str receipt:
         """
-        self.condition = threading.Condition()
+        self.receipt_condition = threading.Condition()
+        self.disconnect_condition = threading.Condition()
         self.receipt = receipt
         self.received = False
+        self.disconnected = False
 
-    def on_receipt(self, headers, body):
+    def on_receipt(self, frame):
         """
         If the receipt id can be found in the headers, then notify the waiting thread.
 
         :param dict headers: headers in the message
         :param body: the message content
         """
-        if 'receipt-id' in headers and headers['receipt-id'] == self.receipt:
-            with self.condition:
+        if "receipt-id" in frame.headers and frame.headers["receipt-id"] == self.receipt:
+            with self.receipt_condition:
                 self.received = True
-                self.condition.notify()
+                self.receipt_condition.notify()
+
+    def on_disconnected(self):
+        with self.disconnect_condition:
+            self.disconnected = True
+            self.disconnect_condition.notify()
 
     def wait_on_receipt(self):
         """
         Wait until we receive a message receipt.
         """
-        with self.condition:
+        with self.receipt_condition:
             while not self.received:
-                self.condition.wait()
-        self.received = False
+                self.receipt_condition.wait()
+            self.received = False
 
+    def wait_on_disconnected(self):
+        """
+        Wait until disconnected.
+        """
+        with self.disconnect_condition:
+            while not self.disconnected:
+                self.disconnect_condition.wait()
 
 class StatsListener(ConnectionListener):
     """
@@ -351,16 +384,19 @@ class StatsListener(ConnectionListener):
         Increment the disconnect count. See :py:meth:`ConnectionListener.on_disconnected`
         """
         self.disconnects += 1
-        log.info("disconnected (x %s)", self.disconnects)
+        logging.info("disconnected (x %s)", self.disconnects)
 
-    def on_error(self, headers, body):
+    def on_error(self, frame):
         """
         Increment the error count. See :py:meth:`ConnectionListener.on_error`
 
         :param dict headers: headers in the message
         :param body: the message content
         """
-        log.info("received an error %s [%s]", body, headers)
+        if logging.isEnabledFor(logging.DEBUG):
+            logging.debug("received an error %s [%s]", frame.body, frame.headers)
+        else:
+            logging.info("received an error %s", frame.body)
         self.errors += 1
 
     def on_connecting(self, host_and_port):
@@ -369,10 +405,10 @@ class StatsListener(ConnectionListener):
 
         :param (str,int) host_and_port: the host and port as a tuple
         """
-        log.info("connecting %s %s (x %s)", host_and_port[0], host_and_port[1], self.connections)
+        logging.info("connecting %s %s (x %s)", host_and_port[0], host_and_port[1], self.connections)
         self.connections += 1
 
-    def on_message(self, headers, body):
+    def on_message(self, frame):
         """
         Increment the message received count. See :py:meth:`ConnectionListener.on_message`
 
@@ -393,7 +429,7 @@ class StatsListener(ConnectionListener):
         """
         Increment the heartbeat timeout. See :py:meth:`ConnectionListener.on_heartbeat_timeout`
         """
-        log.debug("received heartbeat timeout")
+        logging.debug("received heartbeat timeout")
         self.heartbeat_timeouts += 1
 
     def on_heartbeat(self):
@@ -408,115 +444,173 @@ class StatsListener(ConnectionListener):
         errors, etc)
         """
         return '''Connections: %s
+Disconnects: %s
 Messages sent: %s
 Messages received: %s
 Heartbeats received: %s
-Errors: %s''' % (self.connections, self.messages_sent, self.messages, self.heartbeat_count, self.errors)
+Errors: %s''' % (self.connections, self.disconnects, self.messages_sent, self.messages, self.heartbeat_count, self.errors)
 
 
 class PrintingListener(ConnectionListener):
+    def __init__(self, print_to_log=False):
+        self.print_to_log = print_to_log
+
+    def __print(self, msg, *args):
+        if self.print_to_log:
+            logging.info(msg, *args)
+        else:
+            print(msg % args)
+
     def on_connecting(self, host_and_port):
         """
         :param (str,int) host_and_port:
         """
-        print('on_connecting %s %s' % host_and_port)
+        self.__print("on_connecting %s %s", *host_and_port)
 
-    def on_connected(self, headers, body):
+    def on_connected(self, frame):
         """
         :param dict headers:
         :param body:
         """
-        print('on_connected %s %s' % (headers, body))
+        self.__print("on_connected %s %s", frame.headers, frame.body)
 
     def on_disconnected(self):
-        print('on_disconnected')
+        self.__print("on_disconnected")
 
     def on_heartbeat_timeout(self):
-        print('on_heartbeat_timeout')
+        self.__print("on_heartbeat_timeout")
 
-    def on_before_message(self, headers, body):
+    def on_before_message(self, frame):
         """
         :param dict headers:
         :param body:
         """
-        print('on_before_message %s %s' % (headers, body))
-        return headers, body
+        self.__print("on_before_message %s %s", frame.headers, frame.body)
 
-    def on_message(self, headers, body):
+    def on_message(self, frame):
         """
         :param dict headers:
         :param body:
         """
-        print('on_message %s %s' % (headers, body))
+        self.__print("on_message %s %s", frame.headers, frame.body)
 
-    def on_receipt(self, headers, body):
+    def on_receipt(self, frame):
         """
         :param dict headers:
         :param body:
         """
-        print('on_receipt %s %s' % (headers, body))
+        self.__print("on_receipt %s %s", frame.headers, frame.body)
 
-    def on_error(self, headers, body):
+    def on_error(self, frame):
         """
         :param dict headers:
         :param body:
         """
-        print('on_error %s %s' % (headers, body))
+        self.__print("on_error %s %s", frame.headers, frame.body)
 
     def on_send(self, frame):
         """
         :param Frame frame:
         """
-        print('on_send %s %s %s' % (frame.cmd, frame.headers, frame.body))
+        self.__print("on_send %s %s %s", frame.cmd, utils.clean_headers(frame.headers), frame.body)
 
     def on_heartbeat(self):
-        print('on_heartbeat')
+        self.__print("on_heartbeat")
 
 
-class TestListener(StatsListener, WaitingListener):
+class TestListener(StatsListener, WaitingListener, PrintingListener):
     """
     Implementation of StatsListener and WaitingListener. Useful for testing.
     """
-    def __init__(self, receipt=None):
+    def __init__(self, receipt=None, print_to_log=False):
         """
         :param str receipt:
         """
         StatsListener.__init__(self)
+        PrintingListener.__init__(self, print_to_log)
         WaitingListener.__init__(self, receipt)
         self.message_list = []
         self.message_condition = threading.Condition()
         self.message_received = False
         self.heartbeat_condition = threading.Condition()
         self.heartbeat_received = False
-
-    def on_message(self, headers, message):
-        """
-        :param dict headers:
-        :param message:
-        """
-        StatsListener.on_message(self, headers, message)
-        self.message_list.append((headers, message))
-        with self.message_condition:
-            self.message_received = True
-            self.message_condition.notify()
+        self.timestamp = time.strftime("%Y%m%d%H%M%S")
 
     def wait_for_message(self):
         with self.message_condition:
             while not self.message_received:
                 self.message_condition.wait()
-        self.message_received = False
+            self.message_received = False
 
     def get_latest_message(self):
         return self.message_list[-1]
 
-    def on_heartbeat(self):
-        StatsListener.on_heartbeat(self)
-        with self.heartbeat_condition:
-            self.heartbeat_received = True
-            self.heartbeat_condition.notify()
-
     def wait_for_heartbeat(self):
         with self.heartbeat_condition:
             while not self.heartbeat_received:
                 self.heartbeat_condition.wait()
-        self.heartbeat_received = False
+            self.heartbeat_received = False
+
+    def on_connecting(self, host_and_port):
+        StatsListener.on_connecting(self, host_and_port)
+        PrintingListener.on_connecting(self, host_and_port)
+        WaitingListener.on_connecting(self, host_and_port)
+
+    def on_connected(self, frame):
+        StatsListener.on_connected(self, frame)
+        PrintingListener.on_connected(self, frame)
+        WaitingListener.on_connected(self, frame)
+
+    def on_disconnected(self):
+        StatsListener.on_disconnected(self)
+        PrintingListener.on_disconnected(self)
+        WaitingListener.on_disconnected(self)
+
+    def on_heartbeat_timeout(self):
+        StatsListener.on_heartbeat_timeout(self)
+        PrintingListener.on_heartbeat_timeout(self)
+        WaitingListener.on_heartbeat_timeout(self)
+
+    def on_before_message(self, frame):
+        StatsListener.on_before_message(self, frame)
+        PrintingListener.on_before_message(self, frame)
+        WaitingListener.on_before_message(self, frame)
+
+    def on_message(self, frame):
+        """
+        :param dict headers:
+        :param message:
+        """
+        StatsListener.on_message(self, frame)
+        PrintingListener.on_message(self, frame)
+        self.message_list.append((frame.headers, frame.body))
+        with self.message_condition:
+            self.message_received = True
+            self.message_condition.notify()
+
+    def on_receipt(self, frame):
+        StatsListener.on_receipt(self, frame)
+        PrintingListener.on_receipt(self, frame)
+        WaitingListener.on_receipt(self, frame)
+
+    def on_error(self, frame):
+        StatsListener.on_error(self, frame)
+        PrintingListener.on_error(self, frame)
+        WaitingListener.on_error(self, frame)
+
+    def on_send(self, frame):
+        StatsListener.on_send(self, frame)
+        PrintingListener.on_send(self, frame)
+        WaitingListener.on_send(self, frame)
+
+    def on_heartbeat(self):
+        StatsListener.on_heartbeat(self)
+        PrintingListener.on_heartbeat(self)
+        with self.heartbeat_condition:
+            self.heartbeat_received = True
+            self.heartbeat_condition.notify()
+
+    def on_receiver_loop_completed(self, frame):
+        StatsListener.on_receiver_loop_completed(self, frame)
+        PrintingListener.on_receiver_loop_completed(self, frame)
+        WaitingListener.on_receiver_loop_completed(self, frame)
