@@ -1,16 +1,9 @@
 """Provides the 1.0, 1.1 and 1.2 protocol classes.
 """
 
-import uuid
-
-from stomp.backward import encode
-from stomp.constants import *
+import stomp.utils as utils
 from stomp.exception import ConnectFailedException
 from stomp.listener import *
-import stomp.utils as utils
-
-
-log = logging.getLogger('stomp.py')
 
 
 class Protocol10(ConnectionListener):
@@ -25,8 +18,8 @@ class Protocol10(ConnectionListener):
     def __init__(self, transport, auto_content_length=True):
         self.transport = transport
         self.auto_content_length = auto_content_length
-        transport.set_listener('protocol-listener', self)
-        self.version = '1.0'
+        transport.set_listener("protocol-listener", self)
+        self.version = "1.0"
 
     def send_frame(self, cmd, headers=None, body=''):
         """
@@ -59,6 +52,7 @@ class Protocol10(ConnectionListener):
 
         :param str id: identifier of the message
         :param str transaction: include the acknowledgement in the specified transaction
+        :param str receipt: the receipt id
         """
         assert id is not None, "'id' is required"
         headers = {HDR_MESSAGE_ID: id}
@@ -82,7 +76,7 @@ class Protocol10(ConnectionListener):
         """
         headers = utils.merge_headers([headers, keyword_headers])
         if not transaction:
-            transaction = str(uuid.uuid4())
+            transaction = utils.get_uuid()
         headers[HDR_TRANSACTION] = transaction
         self.send_frame(CMD_BEGIN, headers)
         return transaction
@@ -138,10 +132,10 @@ class Protocol10(ConnectionListener):
         :param keyword_headers: any additional headers the broker requires
         """
         if not self.transport.is_connected():
-            log.debug('Not sending disconnect, already disconnected')
+            logging.debug("Not sending disconnect, already disconnected")
             return
         headers = utils.merge_headers([headers, keyword_headers])
-        rec = receipt or str(uuid.uuid4())
+        rec = receipt or utils.get_uuid()
         headers[HDR_RECEIPT] = rec
         self.set_receipt(rec, CMD_DISCONNECT)
         self.send_frame(CMD_DISCONNECT, headers)
@@ -162,12 +156,11 @@ class Protocol10(ConnectionListener):
         headers[HDR_DESTINATION] = destination
         if content_type:
             headers[HDR_CONTENT_TYPE] = content_type
-        body = encode(body)
         if self.auto_content_length and body and HDR_CONTENT_LENGTH not in headers:
             headers[HDR_CONTENT_LENGTH] = len(body)
         self.send_frame(CMD_SEND, headers, body)
 
-    def subscribe(self, destination, id=None, ack='auto', headers=None, **keyword_headers):
+    def subscribe(self, destination, id=None, ack="auto", headers=None, **keyword_headers):
         """
         Subscribe to a destination.
 
@@ -214,13 +207,14 @@ class Protocol11(HeartbeatListener, ConnectionListener):
     :param transport:
     :param (int,int) heartbeats:
     :param bool auto_content_length: Whether to calculate and send the content-length header automatically if it has not been set
+    :param float heart_beat_receive_scale: how long to wait for a heartbeat before timing out, as a scale factor of receive time
     """
-    def __init__(self, transport, heartbeats=(0, 0), auto_content_length=True):
-        HeartbeatListener.__init__(self, heartbeats)
+    def __init__(self, transport, heartbeats=(0, 0), auto_content_length=True, heart_beat_receive_scale=1.5):
+        HeartbeatListener.__init__(self, transport, heartbeats, heart_beat_receive_scale)
         self.transport = transport
         self.auto_content_length = auto_content_length
-        transport.set_listener('protocol-listener', self)
-        self.version = '1.1'
+        transport.set_listener("protocol-listener", self)
+        self.version = "1.1"
 
     def _escape_headers(self, headers):
         """
@@ -228,7 +222,7 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         """
         for key, val in headers.items():
             try:
-                val = val.replace('\\', '\\\\').replace('\n', '\\n').replace(':', '\\c')
+                val = val.replace("\\", "\\\\").replace("\n", "\\n").replace(":", "\\c")
             except:
                 pass
             headers[key] = val
@@ -269,6 +263,7 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         :param str id: identifier of the message
         :param str subscription: the subscription this message is associated with
         :param str transaction: include the acknowledgement in the specified transaction
+        :param str receipt: the receipt id
         """
         assert id is not None, "'id' is required"
         assert subscription is not None, "'subscription' is required"
@@ -293,7 +288,7 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         """
         headers = utils.merge_headers([headers, keyword_headers])
         if not transaction:
-            transaction = str(uuid.uuid4())
+            transaction = utils.get_uuid()
         headers[HDR_TRANSACTION] = transaction
         self.send_frame(CMD_BEGIN, headers)
         return transaction
@@ -352,25 +347,28 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         :param keyword_headers: any additional headers the broker requires
         """
         if not self.transport.is_connected():
-            log.debug('Not sending disconnect, already disconnected')
+            logging.debug("Not sending disconnect, already disconnected")
             return
         headers = utils.merge_headers([headers, keyword_headers])
-        rec = receipt or str(uuid.uuid4())
+        rec = receipt or utils.get_uuid()
         headers[HDR_RECEIPT] = rec
         self.set_receipt(rec, CMD_DISCONNECT)
         self.send_frame(CMD_DISCONNECT, headers)
 
-    def nack(self, id, subscription, transaction=None, receipt=None):
+    def nack(self, id, subscription, transaction=None, receipt=None, **keyword_headers):
         """
         Let the server know that a message was not consumed.
 
         :param str id: the unique id of the message to nack
         :param str subscription: the subscription this message is associated with
         :param str transaction: include this nack in a named transaction
+        :param str receipt: the receipt id
+        :param keyword_headers: any additional headers to send with the nack command
         """
         assert id is not None, "'id' is required"
         assert subscription is not None, "'subscription' is required"
         headers = {HDR_MESSAGE_ID: id, HDR_SUBSCRIPTION: subscription}
+        headers = utils.merge_headers([headers, keyword_headers])
         if transaction:
             headers[HDR_TRANSACTION] = transaction
         if receipt:
@@ -393,12 +391,11 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         headers[HDR_DESTINATION] = destination
         if content_type:
             headers[HDR_CONTENT_TYPE] = content_type
-        body = encode(body)
         if self.auto_content_length and body and HDR_CONTENT_LENGTH not in headers:
             headers[HDR_CONTENT_LENGTH] = len(body)
         self.send_frame(CMD_SEND, headers, body)
 
-    def subscribe(self, destination, id, ack='auto', headers=None, **keyword_headers):
+    def subscribe(self, destination, id, ack="auto", headers=None, **keyword_headers):
         """
         Subscribe to a destination
 
@@ -439,10 +436,11 @@ class Protocol12(Protocol11):
     :param transport:
     :param (int,int) heartbeats:
     :param bool auto_content_length: Whether to calculate and send the content-length header automatically if it has not been set
+    :param float heart_beat_receive_scale: how long to wait for a heartbeat before timing out, as a scale factor of receive time
     """
-    def __init__(self, transport, heartbeats=(0, 0), auto_content_length=True):
-        Protocol11.__init__(self, transport, heartbeats, auto_content_length)
-        self.version = '1.2'
+    def __init__(self, transport, heartbeats=(0, 0), auto_content_length=True, heart_beat_receive_scale=1.5):
+        Protocol11.__init__(self, transport, heartbeats, auto_content_length, heart_beat_receive_scale=heart_beat_receive_scale)
+        self.version = "1.2"
 
     def _escape_headers(self, headers):
         """
@@ -450,7 +448,7 @@ class Protocol12(Protocol11):
         """
         for key, val in headers.items():
             try:
-                val = val.replace('\\', '\\\\').replace('\n', '\\n').replace(':', '\\c').replace('\r', '\\r')
+                val = val.replace("\\", "\\\\").replace("\n", "\\n").replace(":", "\\c").replace("\r", "\\r")
             except:
                 pass
             headers[key] = val
@@ -461,6 +459,7 @@ class Protocol12(Protocol11):
 
         :param str id: identifier of the message
         :param str transaction: include the acknowledgement in the specified transaction
+        :param str receipt: the receipt id
         """
         assert id is not None, "'id' is required"
         headers = {HDR_ID: id}
@@ -470,15 +469,18 @@ class Protocol12(Protocol11):
             headers[HDR_RECEIPT] = receipt
         self.send_frame(CMD_ACK, headers)
 
-    def nack(self, id, transaction=None, receipt=None):
+    def nack(self, id, transaction=None, receipt=None, **keyword_headers):
         """
         Let the server know that a message was not consumed.
 
         :param str id: the unique id of the message to nack
         :param str transaction: include this nack in a named transaction
+        :param str receipt: the receipt id
+        :param keyword_headers: any additional headers to send with the nack command
         """
         assert id is not None, "'id' is required"
         headers = {HDR_ID: id}
+        headers = utils.merge_headers([headers, keyword_headers])
         if transaction:
             headers[HDR_TRANSACTION] = transaction
         if receipt:
